@@ -1,6 +1,6 @@
 package Lemonldap::NG::Portal::Main::Process;
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.0.15';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -8,6 +8,7 @@ use strict;
 use MIME::Base64;
 use POSIX qw(strftime);
 use Lemonldap::NG::Portal::Main::Constants qw(portalConsts);
+use URI;
 
 # Main method
 # -----------
@@ -40,7 +41,7 @@ sub process {
         }
     }
     $self->logger->debug( "Returned " . $self->_formatProcessResult($err) )
-      if ($err);
+      if $err;
     return $err;
 }
 
@@ -51,6 +52,8 @@ sub processHook {
     my $err = PE_OK;
     for my $sub ( @{ $self->hook->{$hookName} } ) {
         if ( ref $sub eq 'CODE' ) {
+
+            # If return code is not PE_OK (0), stop processing
             last if ( $err = $sub->( $req, @args ) );
         }
         else {
@@ -58,8 +61,14 @@ sub processHook {
         }
     }
     if ( $err != PE_OK ) {
-        $self->logger->warn(
-            "Hook $hookName returned " . portalConsts->{$err} );
+        my $msg = "Hook $hookName returned " . portalConsts->{$err};
+        if ( $err > 0 ) {
+            $self->logger->warn($msg);
+        }
+        else {
+            # <0 error codes are normal in some use cases
+            $self->logger->debug($msg);
+        }
     }
     return $err;
 }
@@ -79,7 +88,7 @@ sub _formatProcessResult {
 sub restoreArgs {
     my ( $self, $req ) = @_;
     $req->mustRedirect(1);
-    PE_OK;
+    return PE_OK;
 }
 
 sub importHandlerData {
@@ -87,7 +96,7 @@ sub importHandlerData {
     $req->{sessionInfo} = $req->userData;
     $req->id( $req->sessionInfo->{_session_id} );
     $req->user( $req->sessionInfo->{ $self->conf->{whatToTrace} } );
-    PE_OK;
+    return PE_OK;
 }
 
 # Verify url and confirm parameter
@@ -161,14 +170,22 @@ sub controlUrl {
             delete $req->{urldc};
             return PE_BADURL;
         }
-        my ( $proto, $vhost, $appuri ) = ( $2, $3, $5 );
+        my ( $proto, $vhost, $appuri );
+        if ($tmp) {
+            my $u = URI->new($tmp);
+            if ( $u->scheme =~ /^https?$/ ) {
+                $proto  = $u->scheme;
+                $vhost  = $u->host if $u->can("host");
+                $appuri = $u->path_query;
+            }
+        }
 
         # Try to resolve alias
         my $originalVhost = $self->HANDLER->resolveAlias($vhost);
-        $vhost = $proto . '://' . $originalVhost;
-        $self->logger->debug( "Required URL (param: "
-              . ( $req->param('logout') ? 'HTTP Referer' : 'urldc' )
-              . " | value: $tmp | alias: $vhost)" );
+        $vhost = "$proto://$originalVhost";
+        $self->logger->debug( 'Required URL (param: '
+              . ( $req->param('logout') ? 'HTTP Referer'   : 'urldc' )
+              . ( $tmp ? " | value: $tmp | alias: $vhost)" : ')' ) );
 
         # If the target URL has an authLevel set in config, remember it.
         my $level = $self->HANDLER->getLevel( $req, $appuri, $originalVhost );
@@ -179,11 +196,11 @@ sub controlUrl {
             and !$self->isTrustedUrl($vhost) )
         {
             $self->userLogger->error(
-                    "URL contains a non protected host (param: "
+                    "URL contains an unprotected host (param: "
                   . ( $req->param('logout') ? 'HTTP Referer' : 'urldc' )
                   . " | value: $tmp | alias: $vhost)" );
             delete $req->{urldc};
-            return PE_BADURL;
+            return PE_UNPROTECTEDURL;
         }
 
         $req->env->{urldc} = $req->{urldc};
@@ -191,7 +208,7 @@ sub controlUrl {
         $req->data->{_url} = $req->pdata->{_url} =
           encode_base64( $req->{urldc}, '' );    # Avoid \n or \r
     }
-    PE_OK;
+    return PE_OK;
 }
 
 sub checkLogout {
@@ -200,7 +217,7 @@ sub checkLogout {
         $req->steps(
             [ @{ $self->beforeLogout }, 'authLogout', 'deleteSession' ] );
     }
-    PE_OK;
+    return PE_OK;
 }
 
 sub checkUnauthLogout {
@@ -221,7 +238,7 @@ sub checkUnauthLogout {
         );
         $req->steps( [ sub { PE_LOGOUT_OK } ] );
     }
-    PE_OK;
+    return PE_OK;
 }
 
 sub authLogout {
@@ -297,22 +314,22 @@ sub deleteSession {
         # Redirect on logout page if no other target defined
         if ( !$req->urldc and !$req->postUrl ) {
             $self->logger->debug('No other target defined, redirect on logout');
-            $req->urldc( $req->script_name . "?logout=1" );
+            $req->urldc( $self->buildUrl( { logout => 1 } ) );
         }
     }
     $req->userData( {} );
 
     # Redirect or Post if asked by authLogout
-    if ( $req->urldc and $req->urldc ne $self->conf->{portal} ) {
-        $req->steps( [] );
-        return PE_REDIRECT;
-    }
-
     if ( $req->postUrl ) {
         $req->steps( ['autoPost'] );
         return PE_OK;
     }
 
+    if ( $req->urldc and $req->urldc ne $self->conf->{portal} ) {
+        $req->steps( [] );
+        return PE_REDIRECT;
+    }
+
     # If logout redirects to another URL, just remove next steps for the
     # request so autoRedirect will be called
     if ( $req->{urldc} and $req->{urldc} ne $self->conf->{portal} ) {
@@ -400,7 +417,7 @@ sub authenticate {
     $req->steps( [
             'setSessionInfo',           'setMacros',
             'setPersistentSessionInfo', 'storeHistory',
-            @{ $self->afterData }, sub { PE_BADCREDENTIALS }
+            @{ $self->afterData },      sub { PE_BADCREDENTIALS }
         ]
     );
 
@@ -461,7 +478,6 @@ sub setSessionInfo {
 
     # Call UserDB setSessionInfo
     return $self->_userDB->setSessionInfo($req);
-    PE_OK;
 }
 
 sub setMacros {
@@ -470,7 +486,7 @@ sub setMacros {
         $req->{sessionInfo}->{$_} =
           $self->_macros->{$_}->( $req, $req->sessionInfo );
     }
-    PE_OK;
+    return PE_OK;
 }
 
 sub setGroups {
@@ -488,7 +504,6 @@ sub setPersistentSessionInfo {
         return PE_OK unless ( $key and length($key) );
 
         my $persistentSession = $self->getPersistentSession($key);
-
         if ($persistentSession) {
             $self->logger->debug("Persistent session found for $key");
             foreach my $k ( keys %{ $persistentSession->data } ) {
@@ -500,11 +515,13 @@ sub setPersistentSessionInfo {
             }
         }
     }
-    PE_OK;
+    return PE_OK;
 }
 
 sub setLocalGroups {
     my ( $self, $req ) = @_;
+    $req->{sessionInfo}->{groups}  //= '';
+    $req->{sessionInfo}->{hGroups} //= {};
     foreach ( sort keys %{ $self->_groups } ) {
         if ( $self->_groups->{$_}->( $req, $req->sessionInfo ) ) {
             $req->{sessionInfo}->{groups} .=
@@ -518,7 +535,7 @@ sub setLocalGroups {
         $req->{sessionInfo}->{groups} =~
           s/^$self->{conf}->{multiValuesSeparator}//o;
     }
-    PE_OK;
+    return PE_OK;
 }
 
 sub store {
@@ -545,11 +562,10 @@ sub store {
     foreach my $k ( keys %{ $req->{sessionInfo} } ) {
         next unless defined $req->{sessionInfo}->{$k};
         my $displayValue = $req->{sessionInfo}->{$k};
-        if (    $self->conf->{hiddenAttributes}
-            and $self->conf->{hiddenAttributes} =~ /\b$k\b/ )
-        {
-            $displayValue = '****';
-        }
+        $displayValue = '****'
+          if (  $self->conf->{hiddenAttributes}
+            and $self->conf->{hiddenAttributes} =~ /\b$k\b/ );
+
         $self->logger->debug("Store $displayValue in session key $k");
         $self->_dump($displayValue) if ref($displayValue);
         $infos->{$k} = $req->{sessionInfo}->{$k};
@@ -561,7 +577,7 @@ sub store {
         force => $req->{force},
         info  => $infos
     );
-    return PE_APACHESESSIONERROR unless ($session);
+    return PE_APACHESESSIONERROR unless $session;
 
     # Update current request
     $req->id( $session->id );
@@ -580,7 +596,7 @@ sub store {
               . $req->{sessionInfo}->{_httpSession} );
     }
     $req->refresh(0);
-    PE_OK;
+    return PE_OK;
 }
 
 sub buildCookie {
@@ -614,7 +630,7 @@ sub buildCookie {
           . $ref->{ $self->conf->{whatToTrace} }
           . " successfully authenticated at level $ref->{authenticationLevel}"
     );
-    PE_OK;
+    return PE_OK;
 }
 
 sub secondFactor {
@@ -627,7 +643,7 @@ sub storeHistory {
     if ( $self->conf->{loginHistoryEnabled} ) {
         $self->registerLogin($req);
     }
-    PE_OK;
+    return PE_OK;
 }
 
 1;
